/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.book;

import com.google.common.collect.ComparisonChain;

public abstract class AbstractContactGroup implements ContactGroup, Comparable<AbstractContactGroup> {
	
	public static class Origin {
		private final String origin;

		public Origin(String origin) {
			this.origin = origin;
		}
		
		public String getOrigin() {
			return origin;
		}
	}
	
	public static class Id implements ContactGroup.Id {
		private String id;
		private Origin origin;
		
		public Id(String id, Origin origin) {
			this.id = id;
			this.origin = origin;
		}

		@Override
		public String idAsString() {
			return id;
		}
		
		@Override
		public String originAsString() {
			return origin.getOrigin();
		}
		
		@Override
		public int compareTo(ContactGroup.Id o) {
			return ComparisonChain.start()
				.compare(originAsString(), o.originAsString())
				.compare(idAsString(), o.idAsString())
				.result();
		}
		
	}
	
	private Id id;
	private String displayName;
	
	public AbstractContactGroup(Id id, String displayName) {
		super();
		this.id = id;
		this.displayName = displayName;
	}

	public Id getId() {
		return id;
	}

	public String getDisplayName() {
		return displayName;
	}

	public abstract IBookSource getSource();

	@Override
	public int compareTo(AbstractContactGroup o) {
		return ComparisonChain.start()
				.compare(displayName, o.displayName)
				.result();
	}
}
