/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package org.minig.mailinglist;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.minig.obmsync.exception.ObmSyncConnectionException;
import org.minig.obmsync.service.IMailingListService;
import org.minig.obmsync.service.MailingListService;

import com.google.common.collect.Lists;

import fr.aliasource.webmail.book.AbstractContactGroup;
import fr.aliasource.webmail.book.AbstractContactGroup.Origin;
import fr.aliasource.webmail.book.ContactGroup;
import fr.aliasource.webmail.book.ContactGroup.Id;
import fr.aliasource.webmail.book.IBookSource;
import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.proxy.ProxyConfiguration;

/**
 * 
 * @author "Matthieu Laborie <laborima@gmail.com>"
 *
 */
public class MailingListAddressBook implements IBookSource {

	public static final Origin MAILING_SOURCE_ORIGIN = new Origin("mailingLists");
	public static final Id MAILING_SOURCE_ID = new AbstractContactGroup.Id("mailing", MAILING_SOURCE_ORIGIN);
	
	private Log logger = LogFactory.getLog(getClass());
	
	private class MailingListContactGroup implements ContactGroup {

		private final String userId;
		private final String userPassword;

		public MailingListContactGroup(String userId, String userPassword) {
			this.userId = userId;
			this.userPassword = userPassword;
		}

		@Override
		public List<IMinigContact> findAll() {
			return MailingListAddressBook.this.findAll(userId, userPassword);
		}

		@Override
		public List<IMinigContact> find(String query, int limit) {
			return MailingListAddressBook.this.find(userId, userPassword, query, limit);
		}

		@Override
		public int count() {
			return MailingListAddressBook.this.count(userId, userPassword);
		}

		@Override
		public void insert(List<MinigContact> c) {
		}

		@Override
		public GroupType getType() {
			return GroupType.READ_ONLY;
		}

		@Override
		public Id getId() {
			return MailingListAddressBook.MAILING_SOURCE_ID;
		}

		@Override
		public String getDisplayName() {
			return "Mailing Lists";
		}
		
	}
	

	public MailingListAddressBook() {
	}

	public int count(String userId, String userPassword) {
		try {
			IMailingListService mlService = getMailingListService(userId,userPassword);
			return mlService.listAllMailingList().size();
		} catch (Exception e){
			logger.error(e);
		}
		return 0;
	}

	//TODO use obm-sync instead
	public List<IMinigContact> find(String userId, String userPassword,
			String query, int limit) {
		List<IMinigContact> ret =  findAll( userId,  userPassword);
		for(Iterator<IMinigContact> it = ret.iterator();it.hasNext();  ){
			if(!it.next().getDisplayName().contains(query)){
				it.remove();
			}
		}
		Collections.sort(ret);
		
		return ret.subList(0, Math.min(ret.size(), limit));
	}

	public List<IMinigContact> findAll(String userId, String userPassword) {
		try{
			IMailingListService mlService = getMailingListService(userId,userPassword);
			return mlService.listAllMailingList();
		} catch (Exception e){
			logger.error(e);
		}
		return new ArrayList<IMinigContact>();
	}

	@Override
	public List<? extends ContactGroup> listGroups(String userId, String userPassword) {
		return Lists.newArrayList(new MailingListContactGroup(userId, userPassword));
	}
	
	@Override
	public void shutdown() {
	}

	@Override
	public void init(ProxyConfiguration pcf) {
		logger.info("initializing obm mailing ab.");
	}



	private IMailingListService getMailingListService(String userId, String userPassword)
			throws ObmSyncConnectionException {
		return new MailingListService(userId,userPassword);
	}
	
}
