package org.minig.mailinglist;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;


import org.minig.obmsync.service.MinigMLEmail;
import org.minig.obmsync.service.MinigMailingList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fr.aliasource.utils.DOMUtils;

public class MailingListItemsParser {

	public MinigMailingList parseMailingList(String parameter) throws Exception {
		Document doc = DOMUtils.parse(new ByteArrayInputStream(parameter
				.getBytes()));

		Element root = doc.getDocumentElement();
		return parseMailingList(root);
	}

	public List<MinigMailingList> parseListMailingList(Document doc) {
		Element documentElement = doc.getDocumentElement();
		List<MinigMailingList> ret = new LinkedList<MinigMailingList>();
		NodeList mlsNodeList = documentElement
				.getElementsByTagName("mailingList");
		for (int i = 0; i < mlsNodeList.getLength(); i++) {
			Element e = (Element) mlsNodeList.item(i);
			MinigMailingList ml = parseMailingList(e);
			ret.add(ml);
		}
		return ret;
	}

	public MinigMailingList parseMailingList(Element root) {
		Element mlEle = root;
		if(!mlEle.getTagName().equalsIgnoreCase("mailingList")){
			mlEle = DOMUtils.getUniqueElement(root, "mailingList");	
		}
		MinigMailingList ml = new MinigMailingList();
		String id = mlEle.getAttribute("id");
		if (id != null && id.length() > 0) {
			try {
				ml.setId(Integer.valueOf(id));
			} catch (NumberFormatException e) {
			}
		}
		ml.setName(mlEle.getAttribute("name"));
		ml.addEmails(parseEmails(mlEle));
		return ml;
	}

	private List<MinigMLEmail> parseEmails(Element uniqueElement) {
		String[] attrs = {"id", "label", "address" };
		String[][] values = DOMUtils
				.getAttributes(uniqueElement, "email", attrs);
		List<MinigMLEmail> ret = new ArrayList<MinigMLEmail>(values.length);
		for (String[] p : values) {
			MinigMLEmail e = new MinigMLEmail(p[1], p[2]);
			String id = p[0];
			if (id != null && id.length() > 0) {
				try {
					e.setId(Integer.valueOf(id));
				} catch (NumberFormatException x) {
				}
			}
			ret.add(e);
		}
		return ret;
	}

	public List<MinigMLEmail> parseMailingListEmails(String mailingListEmails) {
		try {
			Document doc = DOMUtils.parse(new ByteArrayInputStream(mailingListEmails
					.getBytes()));
			return parseEmails(doc.getDocumentElement());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new ArrayList<MinigMLEmail>(0);
	}
	
	public List<MinigMLEmail> parseMailingListEmails(Document doc) {
		return parseEmails(doc.getDocumentElement());
	}
}
