package org.minig.mailinglist;

import java.io.ByteArrayOutputStream;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.minig.obmsync.service.MinigMLEmail;
import org.minig.obmsync.service.MinigMailingList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import fr.aliasource.utils.DOMUtils;
import fr.aliasource.webmail.book.IMinigContact;


/**
 * Serializes address book items to XML
 * 
 * @author adrienp
 * 
 */
public class MailingListItemsWriter {
	
	protected Log logger = LogFactory.getLog(getClass());

	public Document getMailingListsAsXML(MinigMailingList... mailingLists) {
		try {
			Document doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/mailingLists.xsd",
					"mailingLists");
			Element root = doc.getDocumentElement();
			for (MinigMailingList ml : mailingLists) {
				appendMailingList(root, ml);
			}
			return doc;
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
		return null;
	}

	public String getMailingListsAsString(MinigMailingList... mailingList) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			Document doc = getMailingListsAsXML(mailingList);
			DOMUtils.serialise(doc, out);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
		return out.toString();
	}

	public Document getMailingListEmailsAsXML(List<MinigMLEmail> emails) {
		Document doc = null;
		try {
			doc = DOMUtils.createDoc(
					"http://www.obm.org/xsd/sync/mailingListEmails.xsd",
					"mailingListEmails");
			Element root = doc.getDocumentElement();
			addEmail(root, emails);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
		return doc;
	}

	public String getMailingListEmailsAsString(List<MinigMLEmail> emails) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			Document doc = getMailingListEmailsAsXML(emails);
			DOMUtils.serialise(doc, out);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		}
		return out.toString();
	}
	
	private void appendMailingList(Element root, MinigMailingList mailingList) {
		if(mailingList instanceof MinigMailingList){
			Element c = root;
			if (!"mailingList".equals(root.getNodeName())) {
				c = DOMUtils.createElement(root, "mailingList");
			}
			if (((MinigMailingList)mailingList).getId() != null) {
				c.setAttribute("id", "" + ((MinigMailingList)mailingList).getId());
			}
			c.setAttribute("name", "" + ((MinigMailingList)mailingList).getName());
			addEmail(c, ((MinigMailingList)mailingList).getMLEmails());
		}
	}

	private void addEmail(Element root, List<MinigMLEmail> emails) {
		Element mle = root;
		if (!"mailingListEmails".equals(root.getNodeName())) {
			mle = DOMUtils.createElement(root, "mailingListEmails");
		}
		for (MinigMLEmail email : emails) {
			Element c = DOMUtils.createElement(mle, "email");
			if (email.getId() != null) {
				c.setAttribute("id", email.getId().toString());
			}
			c.setAttribute("label", email.getLabel());
			c.setAttribute("address", email.getAddress());
		}
	}

}
