/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package org.minig.obm.contacts;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.minig.obmsync.exception.ObmSyncConnectionException;
import org.minig.obmsync.service.BookService;
import org.minig.obmsync.service.IBookService;
import org.minig.obmsync.service.OBMContactGroup;

import fr.aliasource.webmail.book.AddressCollectException;
import fr.aliasource.webmail.book.ContactGroup;
import fr.aliasource.webmail.book.IAddressCollector;
import fr.aliasource.webmail.book.IBookSource;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.proxy.ProxyConfiguration;

public class OBMAddressBook implements IBookSource, IAddressCollector {

	private Log logger = LogFactory.getLog(getClass());

	public OBMAddressBook() {
	}

	@Override
	public List<? extends ContactGroup> listGroups(String userId, String userPassword) {
		try {
			IBookService bookService = getBookService(userId, userPassword);
			List<OBMContactGroup> groups = bookService.listGroups();
			for (OBMContactGroup group: groups) {
				group.setSource(this);
			}
			return groups;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return new ArrayList<ContactGroup>();
	}
	
	private IBookService getBookService(String userId, String userPassword)
			throws ObmSyncConnectionException {
		return new BookService(userId, userPassword);
	}

	@Override
	public void shutdown() {
	}

	@Override
	public void init(ProxyConfiguration pcf) {
		logger.info("initialising obm ab.");
	}

	@Override
	public void collectContacts(String userId, String userPassword,	List<MinigContact> contacts) throws AddressCollectException {
		try {
			IBookService bookService = getBookService(userId, userPassword);
			for (MinigContact contact: contacts) {
				contact.setCollected(true);
			}
			bookService.insert(contacts);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new AddressCollectException(e);
		}
	}

}
