/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package org.minig.obmsync.provider.impl;

import java.util.LinkedList;
import java.util.List;

import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.book.AddressBook;
import org.obm.sync.book.BookType;
import org.obm.sync.book.Contact;
import org.obm.sync.client.book.BookClient;

import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.book.MinigContactFactory;

public class ObmSyncBookProvider  extends AbstractProvider<BookClient> {

	public ObmSyncBookProvider(BookClient client) {
		super(client);
	}

	public void create(MinigContact contact) throws AuthFault, ServerFault {
		client.createContact(token, BookType.contacts, getObmSyncContact(contact));
	}
	
	public List<IMinigContact> find(AddressBook ab, String query, int limit) throws AuthFault, ServerFault {
		return getListContact(client.searchContactInGroup(token, ab, query, limit));
	}

	public List<IMinigContact> getAll(AddressBook ab) throws AuthFault, ServerFault {
		return find(ab, "", Integer.MAX_VALUE);
	}
	
	public int count(AddressBook ab) throws AuthFault, ServerFault {
		return getAll(ab).size();
	}
	
	
	private IMinigContact getMinigContact(org.obm.sync.book.Contact obmSyncContact){
		return new MinigContactFactory().createFrom(obmSyncContact);
	}
	
	private List<IMinigContact> getListContact(List<org.obm.sync.book.Contact> listObmSyncContact){
		List<IMinigContact> contacts = new LinkedList<IMinigContact>();
		for(org.obm.sync.book.Contact c : listObmSyncContact ){
			contacts.add(getMinigContact(c));
		}
		return contacts;
 	}
	
	
	private Contact getObmSyncContact(IMinigContact minigContact){
		return minigContact.adapt(Contact.class);
	}
	
	@SuppressWarnings("unused")
	private List<org.obm.sync.book.Contact> getListObmSyncContact(List<MinigContact> listMinigContact){
		List<org.obm.sync.book.Contact> contacts = new LinkedList<org.obm.sync.book.Contact>();
		for(MinigContact c : listMinigContact ){
			contacts.add(getObmSyncContact(c));
		}
		return contacts;
 	}

	public List<AddressBook> listAddressBooks() throws AuthFault, ServerFault {
		return client.listAllBooks(token);
	}
}
