/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package org.minig.obmsync.provider.impl;

import java.io.InputStream;
import java.util.Date;
import java.util.List;

import org.minig.obmsync.exception.InvalidICSException;
import org.minig.obmsync.service.IEventService;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.ParticipationRole;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.client.calendar.CalendarClient;

import fr.aliasource.utils.FileUtils;

public class ObmSyncCalendarProvider extends AbstractProvider<CalendarClient>{

	public ObmSyncCalendarProvider(CalendarClient client) {
		super(client);
	}

	public CalendarInfo getMyCalendar(String userId) throws ServerFault,
			AuthFault {
		CalendarInfo[] listCalInfo = client.listCalendars(token);
		for (CalendarInfo calInfo : listCalInfo) {
			if (calInfo.getUid().equals(userId.split("@")[0])) {
				return calInfo;
			}
		}
		return null;
	}

	public Event getEventFromExtId(String userId, String uid) throws AuthFault,
			ServerFault {
		return client
				.getEventFromExtId(token, getMyCalendar(userId).getUid(), uid);
	}

	public Event createEvent(String userId, Event event) throws AuthFault,
			ServerFault {
		CalendarInfo myCalendar = getMyCalendar(userId);
		boolean find = false;
		for (Attendee att : event.getAttendees()) {
			if (myCalendar.getMail().equals(att.getEmail())) {
				find = true;
			}
		}
		if (!find) {
			Attendee att = new Attendee();
			att.setEmail(myCalendar.getMail());
			att.setState(ParticipationState.NEEDSACTION);
			att.setRequired(ParticipationRole.OPT);
			event.addAttendee(att);
		}
		logger.info("event to create: " + event + " event.title "
				+ event.getTitle());
		client.createEvent(token, getMyCalendar(userId).getUid(), event, true);
		return event;
	}

	public List<Event> getListEventsFromIntervalDate(String userId, Date start,
			Date end) throws AuthFault, ServerFault {
		return client.getListEventsFromIntervalDate(token, getMyCalendar(userId)
				.getUid(), start, end);
	}

	public void updateParticipationState(String userId, Event event,
			String participationState) throws AuthFault, ServerFault {
		for (Attendee att : event.getAttendees()) {
			if (getMyCalendar(userId).getMail().equals(att.getEmail())) {
				if (IEventService.PARTICIPATION_STATE_ACCEPTED
						.equals(participationState)) {
					att.setState(ParticipationState.ACCEPTED);
				} else if (IEventService.PARTICIPATION_STATE_DECLINED
						.equals(participationState)) {
					att.setState(ParticipationState.DECLINED);
				} else if (IEventService.PARTICIPATION_STATE_NEEDSACTION
						.equals(participationState)) {
					att.setState(ParticipationState.NEEDSACTION);
				}
			}
		}
		client.modifyEvent(token, getMyCalendar(userId).getUid(), event, true, true);
	}

	public String getParticipationState(String userId, Event event)
			throws AuthFault, ServerFault {
		for (Attendee att : event.getAttendees()) {
			if (getMyCalendar(userId).getMail().equals(att.getEmail())) {
				if (ParticipationState.ACCEPTED.equals(att.getState())) {
					return IEventService.PARTICIPATION_STATE_ACCEPTED;
				} else if (ParticipationState.DECLINED.equals(att.getState())) {
					return IEventService.PARTICIPATION_STATE_DECLINED;
				} else if (ParticipationState.DELEGATED.equals(att.getState())) {
					return IEventService.PARTICIPATION_STATE_ACCEPTED;
				} else if (ParticipationState.TENTATIVE.equals(att.getState())) {
					return IEventService.PARTICIPATION_STATE_ACCEPTED;
				} else {
					return IEventService.PARTICIPATION_STATE_NEEDSACTION;
				}
			}
		}
		return "";
	}

	public String getUserEmail() throws AuthFault, ServerFault {
		return client.getUserEmail(token);
	}

	public List<Event> parseICS(InputStream icsFile) throws AuthFault, InvalidICSException{
		String ics = "";
		try {
			ics = FileUtils.streamString(icsFile, true);
			return client.parseICS(token, ics);
		} catch (Exception e) {
			logger.error("ICS: "+ics);
			throw new InvalidICSException(e);
		}
	}
}
