/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package org.minig.obmsync.provider.impl;

import java.util.ArrayList;
import java.util.List;

import org.minig.obmsync.exception.MailingListException;
import org.minig.obmsync.service.MinigMLEmail;
import org.minig.obmsync.service.MinigMailingList;
import org.obm.sync.client.mailingList.MailingListClient;
import org.obm.sync.mailingList.MLEmail;
import org.obm.sync.mailingList.MailingList;

import fr.aliasource.webmail.book.IMinigContact;

public class ObmSyncMailingListProvider extends AbstractProvider<MailingListClient> {


	public ObmSyncMailingListProvider(MailingListClient client) {
		super(client);
	}

	public List<IMinigContact> listAllMailingList() throws MailingListException {
		try {
			List<MailingList> mls = client.listAllMailingList(token);
			return toMinigMailingList(mls);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public MinigMailingList createMailingList(
			MinigMailingList mailingList) throws MailingListException {
		try {
			MailingList ml = client.createMailingList(token, toObm(mailingList));
			return toMinig(ml);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public MinigMailingList modifyMailingList(
			MinigMailingList mailingList) throws MailingListException {
		try {
			MailingList ml = client.modifyMailingList(token, toObm(mailingList));
			return toMinig(ml);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public void removeMailingList( Integer id) throws MailingListException {
		try {
			client.removeMailingList(token, id);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public MinigMailingList getMailingListFromId( Integer id) throws MailingListException {
		try {
			MailingList ml = client.getMailingListFromId(token, id);
			return toMinig(ml);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public List<MinigMLEmail> addEmails( Integer mailingListId,
			List<MinigMLEmail> email) throws MailingListException {
		try {
			List<MLEmail> emails = client.addEmails(token, mailingListId, toObmMLEmail(email));
			return toMinigMLEmail(emails);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}

	public void removeEmail( Integer mailingListId,
			Integer emailId) throws MailingListException {
		try {
			client.removeEmail(token,mailingListId, emailId);
		} catch (Exception e) {
			throw new MailingListException(e);
		}
	}
	
	private List<IMinigContact> toMinigMailingList(List<MailingList> mls) {
		List<IMinigContact> ret = new ArrayList<IMinigContact>(mls.size());
		for(MailingList ml : mls){
			ret.add(toMinig(ml));
		}
		return ret;
	}
	
	private List<MinigMLEmail> toMinigMLEmail(List<MLEmail> emails) {
		List<MinigMLEmail> ret = new ArrayList<MinigMLEmail>(emails.size());
		for(MLEmail ml : emails){
			ret.add(toMinig(ml));
		}
		return ret;
	}
	
	private MinigMailingList toMinig(MailingList ml) {
		MinigMailingList ret = new MinigMailingList();
		ret.setId(ml.getId());
		ret.setName(ml.getName());
		ret.addEmails(toMinigMLEmail(ml.getEmails()));
		return ret;
	}

	private MinigMLEmail toMinig(MLEmail ml) {
		MinigMLEmail ret = new MinigMLEmail(ml.getLabel(), ml.getAddress());
		ret.setId(ml.getId());
		return ret;
	}
	
	private List<MLEmail> toObmMLEmail(List<MinigMLEmail> emails) {
		List<MLEmail> ret = new ArrayList<MLEmail>(emails.size());
		for(MinigMLEmail ml : emails){
			ret.add(toObm(ml));
		}
		return ret;
	}
	
	private MailingList toObm(MinigMailingList ml) {
		MailingList ret = new MailingList();
		ret.setId(ml.getId());
		ret.setName(ml.getName());
		ret.addEmails(toObmMLEmail(ml.getMLEmails()));
		return ret;
	}

	private MLEmail toObm(MinigMLEmail ml) {
		MLEmail ret = new MLEmail(ml.getLabel(), ml.getAddress());
		ret.setId(ml.getId());
		return ret;
	}
	
	
	
}
