package org.minig.obmsync.service;

import java.util.List;

import org.minig.obmsync.exception.ObmSyncConnectionException;
import org.minig.obmsync.provider.impl.ObmSyncBookProvider;
import org.minig.obmsync.provider.impl.ObmSyncProviderFactory;
import org.obm.sync.book.AddressBook;

import com.google.common.collect.Lists;

import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.common.IAccount;
import fr.aliasource.webmail.common.LoginUtils;

public class BookService implements IBookService {
	
	private ObmSyncBookProvider provider;

	private String userId;
	private String userPassword;

	public BookService(IAccount ac) throws ObmSyncConnectionException {
		this(LoginUtils.lat(ac), ac.getUserPassword());
	}

	public BookService(String userId, String userPassword)
			throws ObmSyncConnectionException {
		this.userId = userId;
		this.userPassword = userPassword;
		provider = ObmSyncProviderFactory.getBookProvider(userId);
	}

	@Override
	public int count(OBMContactGroup group) throws Exception {
		try {
			login();
			return provider.count(group.getBook());
		} finally {
			logout();
		}
	}

	@Override
	public List<IMinigContact> find(OBMContactGroup group, String query, int limit) throws Exception {
		try {
			login();
			return provider.find(group.getBook(), query, limit);
		} finally {
			logout();
		}
	}

	@Override
	public void insert(MinigContact c) throws Exception {
		try {
			login();
			provider.create(c);
		} finally {
			logout();
		}
	}

	@Override
	public void insert(List<MinigContact> cl) throws Exception {
		try {
			login();
			for (MinigContact c : cl) {
				provider.create(c);
			}
		} finally {
			logout();
		}
	}

	@Override
	public List<IMinigContact> findAll(OBMContactGroup group) throws Exception {
		try {
			login();
			return provider.getAll(group.getBook());
		} finally {
			logout();
		}
	}

	@Override
	public List<OBMContactGroup> listGroups() throws Exception {
		try {
			login();
			List<AddressBook> books = provider.listAddressBooks();
			List<OBMContactGroup> groups = Lists.newArrayList();
			for (AddressBook book: books) {
				groups.add(new OBMContactGroup(this, book));
			}
			return groups;
		} finally {
			logout();
		}
	}

	
	private void logout() {
		provider.logout();
	}

	private void login() {
		provider.login(userId, userPassword);
	}
	
	public String getUserId() {
		return userId;
	}

}
