package org.minig.obmsync.service;

import java.util.List;

import org.minig.obmsync.exception.MailingListException;
import org.minig.obmsync.exception.ObmSyncConnectionException;
import org.minig.obmsync.provider.impl.ObmSyncMailingListProvider;
import org.minig.obmsync.provider.impl.ObmSyncProviderFactory;

import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.common.IAccount;
import fr.aliasource.webmail.common.LoginUtils;

public class MailingListService implements IMailingListService {

	private ObmSyncMailingListProvider provider;

	private String userId;
	private String userPassword;

	public MailingListService(IAccount ac) throws ObmSyncConnectionException {
		this(LoginUtils.lat(ac), ac.getUserPassword());
	}

	public MailingListService(String userId, String userPassword)
			throws ObmSyncConnectionException {
		this.userId = userId;
		this.userPassword = userPassword;
		provider = ObmSyncProviderFactory.getMailingListProvider(userId);
	}

	private void logout() {
		provider.logout();
	}

	private void login() {
		provider.login(userId, userPassword);
	}

	public String getUserId() {
		return userId;
	}

	@Override
	public List<IMinigContact> listAllMailingList()
			throws MailingListException {
		try {
			login();
			return provider.listAllMailingList();
		} finally {
			logout();
		}
	}

	@Override
	public MinigMailingList createMailingList(MinigMailingList mailingList)
			throws MailingListException {
		try {
			login();
			return provider.createMailingList( mailingList);
		} finally {
			logout();
		}
	}

	@Override
	public MinigMailingList modifyMailingList(MinigMailingList mailingList)
			throws MailingListException {
		try {
			login();
			return provider.modifyMailingList( mailingList);
		} finally {
			logout();
		}
	}

	@Override
	public void removeMailingList(Integer id) throws MailingListException {
		try {
			login();
			provider.removeMailingList( id);
		} finally {
			logout();
		}
	}

	@Override
	public MinigMailingList getMailingListFromId(Integer id)
			throws MailingListException {
		try {
			login();
			return provider.getMailingListFromId( id);
		} finally {
			logout();
		}
	}

	@Override
	public List<MinigMLEmail> addEmails(Integer mailingListId,
			List<MinigMLEmail> emails) throws MailingListException {
		try {
			login();
			return provider.addEmails( mailingListId, emails);
		} finally {
			logout();
		}
	}

	@Override
	public void removeEmail(Integer mailingListId, Integer emailId)
			throws MailingListException {
		try {
			login();
			provider.removeEmail( mailingListId, emailId);
		} finally {
			logout();
		}
	}

}
