package org.minig.obmsync.service;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.book.Contact;
import org.obm.sync.book.Email;

import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigEmail;

public class MinigMailingList implements IMinigContact {
	
	private static Log logger = LogFactory.getLog(MinigMailingList.class);

	public static String MAILING_KEY = "MailingList-";
	
	private Integer id;
	private String name;
	private List<MinigMLEmail> mlEmails;
	
	public MinigMailingList() {
		mlEmails = new LinkedList<MinigMLEmail>();
	}

	public MinigMailingList(String name, Integer id) {
		this.name = name;
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}
	

	public void addEmail(MinigMLEmail email) {
		this.mlEmails.add(email);
	}
	

	public void addEmails(List<MinigMLEmail> emails) {
		this.mlEmails.addAll(emails);
	}
	
	
	
	
	/**
	 * @return the mLemails
	 */
	public List<MinigMLEmail> getMLEmails() {
		return mlEmails;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((mlEmails == null) ? 0 : mlEmails.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MinigMailingList other = (MinigMailingList) obj;
		if (mlEmails == null) {
			if (other.mlEmails != null)
				return false;
		} else if (!mlEmails.equals(other.mlEmails))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public String toString() {
		StringBuilder ret = new StringBuilder();
		ret.append("MailingList ");
		ret.append(name);
		ret.append("' (");
		ret.append(id);
		ret.append(")\n");
		for(MinigMLEmail e : mlEmails){
			ret.append("	");
			ret.append(e.toString());
			ret.append("\n");
		}
		
		return ret.toString();
	}

	@Override
	public int compareTo(IMinigContact o) {
		return o.getDisplayName().compareTo(name);
	}

	@Override
	public String getDisplayName() {
		return name;
	}


	@SuppressWarnings("unchecked")
	public <T> T adapt(Class<T> target) {
		if (target.equals(Contact.class)) {
			Contact ret = new Contact();
			ret.setCollected(false);
			ret.setFirstname(getDisplayName());
			ret.setLastname(MAILING_KEY+id);
			for(MinigMLEmail email : mlEmails){
				ret.addEmail(email.getLabel(), new Email(email.getAddress()));
			}
			return (T) ret;
		}
		logger.warn("Cannot cast MinigContact to type " + target);
		return null;
	}

	
	@Override
	public Map<String, MinigEmail> getEmails() {
		Map<String, MinigEmail> ret = new HashMap<String, MinigEmail>();
		for(MinigMLEmail email : mlEmails){
			ret.put(email.getLabel(), new MinigEmail(email.getAddress()));
		}
		return ret;
	}

	@Override
	public boolean showInCompletion() {
		return false;
	}
}
