package org.minig.obmsync.service;

import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.book.AddressBook;

import fr.aliasource.webmail.book.AbstractContactGroup;
import fr.aliasource.webmail.book.IBookSource;
import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;

public class OBMContactGroup extends AbstractContactGroup {

	private static final Origin origin = new Origin("obm_database");
	private final Log logger = LogFactory.getLog(getClass());
	private final BookService service;
	private IBookSource source;
	private final AddressBook book;
	
	public OBMContactGroup(BookService bookService, AddressBook book) {
		super(new Id(String.valueOf(book.getUid()), origin), book.getName());
		this.service = bookService;
		this.book = book;
	}
	
	@Override
	public List<IMinigContact> findAll() {
		try {
			return service.findAll(this);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return new LinkedList<IMinigContact>();
	}

	@Override
	public List<IMinigContact> find(String query, int limit) {
		try {
			List<IMinigContact> ret = service.find(this, query, limit);
			
			logger.info(service.getUserId() + ", query: " + query + ", " + ", book: " + this.book.getName()
					+ " => " + ret.size() + " contacts.");
			
			return ret;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		
		return new LinkedList<IMinigContact>();
	}

	@Override
	public int count() {
		try {
			return service.count(this);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}

		return 0;
	}

	@Override
	public void insert(List<MinigContact> c) throws Exception {
		service.insert(c);
	}

	@Override
	public GroupType getType() {
		return GroupType.READ_WRITE;
	}

	public void setSource(IBookSource source) {
		this.source = source;
	}
	
	@Override
	public IBookSource getSource() {
		return source;
	}
	
	public AddressBook getBook() {
		return book;
	}
	
}