package fr.aliasource.webmail.client;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HandlesAllKeyEvents;
import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.TextBox;

import fr.aliasource.webmail.client.ctrl.WebmailController;
import fr.aliasource.webmail.client.shared.CloudyFolder;
import fr.aliasource.webmail.client.shared.Folder;
import fr.aliasource.webmail.client.widget.ClearButton;

public class LabelsPanel extends FlexTable implements IFolderSelectionListener {

	private Map<Folder, LabelPresenter> widgets;
	private Anchor showMore;
	private boolean unreadOnly;
	private IFolderClickHandlerFactory clickFactory;
	private TextBox filterField;
	private FlexTable main;
	private boolean withDnD;
	private boolean displayUnreadFolder;
	private SimplePanel emptyLabel;
	private ScrollPanel folderPanel;
	private HorizontalPanel filterPanel;
	private Folder selectFolder;

	public LabelsPanel() {
		this(true, true, true, new IFolderClickHandlerFactory() {
			@Override
			public ClickHandler createHandler(final Folder f) {
				ClickHandler ret = new ClickHandler() {
					public void onClick(ClickEvent event) {
						WebmailController.get().getSelector().select(f);
					}
				};
				return ret;
			}
		});
	}

	public LabelsPanel(boolean editable, boolean unreadOnly, boolean withDnD,
			IFolderClickHandlerFactory clickFactory) {
		this.withDnD = withDnD;
		this.unreadOnly = unreadOnly;
		this.clickFactory = clickFactory;
		displayUnreadFolder = true;
		widgets = new HashMap<Folder, LabelPresenter>();
		addStyleName("whiteBackground");
		addStyleName("labelsPanel");
		setWidth("100%");

		filterPanel = new HorizontalPanel();
		filterField = new TextBox();
		filterField.setWidth("100%");
		createFilterHandlers();
		filterPanel.add(filterField);
		filterPanel.setCellWidth(filterField, "100%");

		ClearButton clear = new ClearButton();
		clear.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				filterField.setText("");
				applyFilter();
			}
		});
		emptyLabel = new SimplePanel();
		DOM.setStyleAttribute(emptyLabel.getElement(), "display", "none");
		emptyLabel.add(new Label(I18N.strings.noUnreadMessages()));

		
		getFlexCellFormatter().setColSpan(0, 0, 2);

		filterPanel.add(clear);
		filterPanel.setCellVerticalAlignment(clear,
				HasVerticalAlignment.ALIGN_MIDDLE);
		try {
			DOM.setStyleAttribute(filterPanel.getElement(), "tableLayout",
					"auto");
		} catch (Throwable t) {
			GWT.log("tableLayout inherit fails on ie :/", null);
		}
		setWidget(0, 0, filterPanel);
		setWidget(1, 0, emptyLabel);
		getFlexCellFormatter().setColSpan(1, 0, 2);
		main = new FlexTable();
		folderPanel = new ScrollPanel(main);
		folderPanel.setStyleName("labelsList");
		DOM.setStyleAttribute(folderPanel.getElement(), "overflowY", "scroll");
		DOM.setStyleAttribute(folderPanel.getElement(), "overflowX", "hidden");
		setWidget(2, 0, folderPanel);
		getFlexCellFormatter().setColSpan(2, 0, 2);

		HorizontalPanel bottomActions = new HorizontalPanel();
		bottomActions.setStyleName("bottomActions");
		if (unreadOnly) {
			showMore = new Anchor(I18N.strings.seeAllFolders());
			showMore.setTitle(I18N.strings.unreadFilteringTip());
			showMore.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					displayUnreadFolder = !displayUnreadFolder;
					showMore.setText(displayUnreadFolder ? I18N.strings
							.seeAllFolders() : I18N.strings.seeUnreadFolders());
					for (Folder f : widgets.keySet()) {
						LabelPresenter lp = widgets.get(f);
						lp.setUnreadOnly(!lp.isUnreadOnly());
					}
					update();
				}
			});
			bottomActions.add(showMore);
		}

		// if (editable) {
		// Anchor edit = new Anchor(I18N.strings.editFolders());
		// edit.addClickHandler(new ClickHandler() {
		// @Override
		// public void onClick(ClickEvent event) {
		// WebmailController.get().getView().showFolderSettings();
		// }
		// });
		// bottomActions.add(edit);
		// bottomActions.setCellHorizontalAlignment(edit,
		// HorizontalPanel.ALIGN_RIGHT);
		// }

		// breaks IE
		try {
			DOM.setStyleAttribute(bottomActions.getElement(), "tableLayout",
					"auto");
		} catch (Throwable t) {
			GWT.log("fails on ie", null);
		}
		setWidget(3, 0, bottomActions);
		getFlexCellFormatter().setColSpan(3, 0, 2);

		WebmailController.get().getSelector().addListener(this);
	}

	private void applyFilter() {
		for (Folder f : widgets.keySet()) {
			LabelPresenter lp = widgets.get(f);
			lp.applyFilter(filterField.getText());
		}
	}

	private void createFilterHandlers() {
		class FilterFieldHandlers extends HandlesAllKeyEvents implements
				ValueChangeHandler<String> {

			@Override
			public void onValueChange(ValueChangeEvent<String> event) {
				applyFilter();
			}

			@Override
			public void onKeyDown(KeyDownEvent event) {
				applyFilter();
			}

			@Override
			public void onKeyUp(KeyUpEvent event) {
				applyFilter();
			}

			@Override
			public void onKeyPress(KeyPressEvent event) {
				applyFilter();
			}

		}
		;
		FilterFieldHandlers ffh = new FilterFieldHandlers();
		ffh.addKeyHandlersTo(filterField);
		filterField.addValueChangeHandler(ffh);
	}

	public void destroy() {
		WebmailController.get().getSelector().removeListener(this);
		for (LabelPresenter lp : widgets.values()) {
			lp.destroy();
		}
		widgets.clear();
		main.clear();
	}

	@Override
	public void folderSelected(Folder f) {
		if (this.selectFolder != null) {
			LabelPresenter lb = this.widgets.get(selectFolder);
			if (lb != null) {
				lb.setSelected(false);
			}
		}
		LabelPresenter selectLabel = widgets.get(f);
		if (selectLabel != null) {
			selectLabel.setSelected(true);
		}
		this.selectFolder = f;
	}

	@Override
	public void foldersChanged(Folder[] folders) {
		Map<Folder, LabelPresenter> cp = new HashMap<Folder, LabelPresenter>(
				this.widgets);
		Arrays.sort(folders, new FolderComparator());

		for (LabelPresenter lp : widgets.values()) {
			lp.destroy();
		}
		main.clear();
		widgets.clear();
		int idx = 0;
		for (Folder f : folders) {
			LabelPresenter lb = cp.get(f);
			int unreadCount = lb != null ? lb.getUnreadCount() : 0;
			boolean selected = lb != null ? lb.isSelected() : false;
			createWidget(idx++, f, unreadCount, selected);
		}
		HTML spacer = new HTML("&nbsp;");
		main.setWidget(idx, 0, spacer);
		main.getCellFormatter().setHeight(idx, 0, "100%");
	}

	private void createWidget(int idx, Folder f, int unreadCount,
			Boolean isSelected) {
		if (!withDnD || !WebmailController.get().isSystemFolder(f.getName())) {
			LabelPresenter lw = new LabelPresenter(f, idx,
					main.getRowFormatter(), withDnD);
			widgets.put(f, lw);
			main.setWidget(idx, 0, lw.getColor());
			main.setWidget(idx, 1, lw.getLink());
			main.getColumnFormatter().setWidth(0, "20px");
			if (clickFactory != null) {
				lw.registerClickHandler(clickFactory.createHandler(f));
			}
			lw.setUnreadCount(unreadCount);
			lw.setUnreadOnly(displayUnreadFolder && unreadOnly);
			lw.setSelected(isSelected);
		}
	}

	@Override
	public void unreadCountChanged(CloudyFolder[] cloudyFolders) {
		Set<Folder> cp = new HashSet<Folder>(this.widgets.keySet());
		cp.removeAll(Arrays.asList(cloudyFolders));
		if (cp.size() > 0) {
			foldersChanged(cloudyFolders);
		} else {
			for (CloudyFolder cloudyFolder : cloudyFolders) {
				LabelPresenter p = widgets.get(cloudyFolder);
				if (p != null) {
					p.setUnreadCount(cloudyFolder.getUnreadCount());
					cp.remove(cloudyFolder);
				}
			}
		}
		update();
	}

	private void update() {
		if (unreadOnly) {
			boolean folder = false;
			for (LabelPresenter lp : widgets.values()) {
				if (lp.isVisible()) {
					folder = true;
					break;
				}
			}
			if (folder) {
				DOM.setStyleAttribute(emptyLabel.getElement(), "display",
						"none");
				DOM.setStyleAttribute(folderPanel.getElement(), "display",
				"block");
				DOM.setStyleAttribute(filterPanel.getElement(), "display",
				"block");
			} else {
				DOM.setStyleAttribute(folderPanel.getElement(), "display",
						"none");
				
				DOM.setStyleAttribute(emptyLabel.getElement(), "display",
						"block");
				if(filterField.getValue()==null || filterField.getValue().length()==0 ){
					DOM.setStyleAttribute(filterPanel.getElement(), "display",
						"none");
				} else {
					DOM.setStyleAttribute(filterPanel.getElement(), "display",
					"block");
				}
			}
			
			
		}
	}

}
