/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.client.addressbook;

import java.util.Set;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.Label;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.shared.ContactGroup;

/**
 * The clickable list of contact groups on the left of the addressbook
 * 
 * @author tom
 * 
 */
public class ContactGroups extends Grid  {

	private int selected = -1;
	private View ui;
	private ContactGroup AllGroup;
	private ContactGroupsDelegate delegate;
	
	interface ContactGroupsDelegate {

		void groupSelected(ContactGroup group);
		
	}
	


	/**
	 * @param delegate the delegate to set
	 */
	public void setDelegate(ContactGroupsDelegate delegate) {
		this.delegate = delegate;
	}

	public ContactGroups(View ui) {
		super(1, 2);
		this.ui = ui;
		AllGroup = null;
		getCellFormatter().setWidth(0, 0, "100%");
		setWidget(0, 0, new Label("Loading..."));
		setCellSpacing(0);
		setStyleName("addressBookList");
	}

	public void updateGrid(final Set<ContactGroup> groups) {
		clear();
		resizeRows(groups.size() + 2);

		// Header
		setWidget(0, 0, new Label(I18N.strings.contactGroup()));
		getRowFormatter().setStyleName(0, "addressBookHeader");

		// Group List
		int i = 0;
		for (ContactGroup group: groups) {
			final int row = ++i;
			String label = group.getDisplayName();
		
			
			Label groupLabel = new Label(label, true);
			Label groupCount = new Label(Integer.toString(group.getSize()), false);
			ClickHandler handler = createClickListener(row, groups, group);
			groupLabel.addClickHandler(handler);
			groupCount.addClickHandler(handler);
			setWidget(row, 0, groupLabel);
			setWidget(row, 1, groupCount);
			getRowFormatter().setStyleName(row, "addressBookItem");
			if (AllGroup == null) {
				AllGroup = group;
			}
		}
		
		
		// Search Results
		setWidget(groups.size() + 1, 0, new Label(I18N.strings
				.searchContactResults(), true));
		getRowFormatter().setVisible(groups.size() + 1, false);
	}

	private ClickHandler createClickListener(final int row,
			final Set<ContactGroup> groups, final ContactGroup cg) {
		return new ClickHandler() {
			public void onClick(ClickEvent sender) {
				getRowFormatter().addStyleName(row, "selectedContactGroup");
				if (selected >= 0) {
					getRowFormatter().removeStyleName(selected,
							"selectedContactGroup");
				}

				// Reset contact list
				ui.getAddressBook().getContactList().reset();

				// Hide "Search Results"
				if (selected == groups.size() + 1) {
					getRowFormatter().setVisible(groups.size() + 1, false);
				}
				// Reset display panel
				ui.getAddressBook().getContactDisplay().reset();
				setSelected(row);
				delegate.groupSelected(cg);

				// Reset search field
				ui.getAddressBook().clearSearchField();
			}
		};
	}


	public int getSelected() {
		return selected;
	}

	public void setSelected(int row) {
		selected = row;
		getRowFormatter().addStyleName(row, "selectedContactGroup");
	}

	public void removeSelected(int row) {
		getRowFormatter().removeStyleName(row, "selectedContactGroup");
	}

	public ContactGroup getAllGroup() {
		return AllGroup;
	}

}
