package fr.aliasource.webmail.client.addressbook;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.shared.DisplayNameUtils;
import fr.aliasource.webmail.client.shared.EmailAddress;
import fr.aliasource.webmail.client.shared.book.UIMailingList;
import fr.aliasource.webmail.client.shared.book.UiContact;
import fr.aliasource.webmail.client.shared.book.UiEmail;
import fr.aliasource.webmail.client.widget.MinigTextBox;
import fr.aliasource.webmail.client.widget.RecipientsPanel;

/**
 * @author mattthieu
 * 
 */
public class MailingListDisplay extends Composite {

	private UIMailingList mailing;
	private MailingListDisplayDelegate delegate;
	
	@UiField
	MinigTextBox nameField;
	@UiField
	RecipientsPanel recipients;
	@UiField
	Button save;
	@UiField
	Button cancel;
	@UiField
	Button delete;
	@UiField
	DivElement messagesDiv;
	
	@UiField
    DivElement errorsDiv;

    @UiField
    Label editTitle;

    @UiField
    Label createTitle;
    
    @UiField
    DivElement nameLabel;
    
    @UiField
    DivElement recipientsLabel;
	
	

	interface Binder extends UiBinder<Widget, MailingListDisplay> {

	}
	
	private static final Binder BINDER = GWT.create(Binder.class);
	


	
	interface MailingListDisplayDelegate {

		void save(UIMailingList mailing);

		void delete(UIMailingList mailing);

		void cancel();
		
	}
	

	/**
	 * @param delegate the delegate to set
	 */
	public void setDelegate(MailingListDisplayDelegate delegate, View ui) {
		this.delegate = delegate;
		recipients.setDelegate(ui);
		
	}
	

	public void showErrors(List<String> errors) {
	        SafeHtmlBuilder b = new SafeHtmlBuilder();
	        for (String error : errors) {
	            b.appendEscaped(error).appendHtmlConstant("<br>");
	        }
	        this.errorsDiv.setInnerHTML(b.toSafeHtml().asString());
    }
	
	public void showMessages( List<String> ms) {
        SafeHtmlBuilder b = new SafeHtmlBuilder();
        for (String message : ms) {
            b.appendEscaped(message).appendHtmlConstant("<br>");
        }
        this.messagesDiv.setInnerHTML(b.toSafeHtml().asString());
        Timer timer = new Timer() { 

			@Override
			public void run() {
				messagesDiv.setInnerHTML("");
				
			}}; 
         timer.schedule(5000); 
}
	

	public MailingListDisplay() {
		initWidget(BINDER.createAndBindUi(this));
		reset();
		this.cancel.setText(I18N.strings.cancel());
		this.delete.setText(I18N.strings.delete());
		this.recipientsLabel.setInnerText(I18N.strings.participants());
		this.nameLabel.setInnerText(I18N.strings.mailingName());
		this.editTitle.setText(I18N.strings.editMailingTitle());
		this.createTitle.setText(I18N.strings.createMailingTitle());
	}

	 @UiHandler("cancel")
	 void handleCancelClick(ClickEvent e) {
	   delegate.cancel();

	  }
	 
	 @UiHandler("save")
	 void handleSaveClick(ClickEvent e) {
		 if(mailing==null){
			 mailing = new UIMailingList();
		 }
		 mailing.setName(nameField.getValue());
		 mailing.setEmails( recipients.getRecipients());
		 delegate.save(mailing);
	 }
	 
	 
	 @UiHandler("delete")
	 void handleDeleteClick(ClickEvent e) {
		 delegate.delete(mailing);
	 }

	 

	public  void reset() {
		this.mailing=null;
		this.recipients.clearText();
		this.nameField.clear();
		this.messagesDiv.setInnerHTML("");
    	this.errorsDiv.setInnerHTML("");
	}
	
	 public void setCreating(boolean creating) {
	        if (creating) {
	            editTitle.setVisible(false);
	            createTitle.setVisible(true);
	        	save.setText(I18N.strings.create());
	            delete.setVisible(false);
	        } else {
	            editTitle.setVisible(true);
	            createTitle.setVisible(false);
	            save.setText(I18N.strings.save());
	            delete.setVisible(true);
	        }
	  }
	

	public void setData(UIMailingList mailingList) {
		this.mailing = mailingList;
		this.recipients.setRecipients(mailingList.getEmails());
		this.nameField.setText(mailing.getDisplayName());
	}

	public void setData(Set<UiContact> contacts) {
		List<EmailAddress> addrs = new ArrayList<EmailAddress>();
		for(UiContact contact : contacts ){
			for (UiEmail e : contact.getEmails().values()) {
				if (e.getEmail() != null && !e.getEmail().isEmpty()) {
						addrs.add(new EmailAddress(DisplayNameUtils.formatName(contact, e.getEmail()), e
								.getEmail()));
					
				}
			}
			
		}
		this.recipients.setRecipients(addrs);
	}
	

	

}
