package fr.aliasource.webmail.client.composer.contacts;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

import fr.aliasource.webmail.client.shared.book.UiDisplayEmail;

public class ChooseContactWidget extends Composite implements
		HasSelectContactHandlers {

	private final static String CSS_SELECT_ITEM = "ccwRowSelected";

	private static final ChooseContactsIcone icons = GWT
			.create(ChooseContactsIcone.class);

	interface Binder extends UiBinder<Widget, ChooseContactWidget> {
	}

	private static final Binder BINDER = GWT.create(Binder.class);

	@UiField
	HTMLPanel rootPanel;
	@UiField
	Image selectedImage;
	@UiField
	Label displayNameLabel;
	@UiField
	Label emailLabel;

	private UiDisplayEmail contact;
	private boolean isSelect = false;

	public ChooseContactWidget(UiDisplayEmail contact) {
		initWidget(BINDER.createAndBindUi(this));
		initRootPanel();
		selectedImage.setResource(icons.selected());
		this.contact = contact;

		if (contact.getDisplayName() != null
				&& !contact.getDisplayName().isEmpty()) {
			displayNameLabel.setText(contact.getDisplayName());
			if(!contact.isMailingList()){
				emailLabel.setText(contact.getEmail());
			}
		} else {
			displayNameLabel.setText(contact.getEmail());
		}
		selectedImage.addStyleName("ccwRowImage");
		update();
	}

	private void initRootPanel() {
		rootPanel.addDomHandler(new MouseOverHandler() {
			@Override
			public void onMouseOver(MouseOverEvent arg0) {
				rootPanel.addStyleName("ccwRowOver");
			}
		}, MouseOverEvent.getType());
		rootPanel.addDomHandler(new MouseOutHandler() {
			@Override
			public void onMouseOut(MouseOutEvent arg0) {
				rootPanel.removeStyleName("ccwRowOver");
			}
		}, MouseOutEvent.getType());

		rootPanel.addDomHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent arg0) {
				isSelect = !isSelect;
				update();
				fireEvent(new SelectContactEvent(contact,
						isSelect ? SelectContactEvent.Selection.Select
								: SelectContactEvent.Selection.Deselect));
			}
		}, ClickEvent.getType());
	}

	private void update() {
		if (isSelect) {
			selectedImage.addStyleName(CSS_SELECT_ITEM);
		} else {
			selectedImage.removeStyleName(CSS_SELECT_ITEM);
		}
	}

	public void setSeleted(boolean select) {
		this.isSelect = select;
		update();
	}

	@Override
	public HandlerRegistration addSelectContactHandler(
			SelectContactHandler handler) {
		return addHandler(handler, SelectContactEvent.getType());
	}

	public boolean applyFilter(String filtre) {
		boolean match = false;
		if (filtre != null && !filtre.isEmpty()) {
			Set<String> set = new HashSet<String>();
			set.add(contact.getDisplayName().toLowerCase());
			if(!contact.isMailingList()){
				set.add(contact.getEmail().toLowerCase());
			}
			if(contact.getDisplayName().contains(" ")){
				set.addAll(Arrays.asList(contact.getDisplayName().toLowerCase().split(" ")));
			}
			String t = filtre.toLowerCase();
			for (String s : set) {
				match = s.startsWith(t);
				setVisible(match);
				if (match) {
					break;
				}
			}
		} else {
			setVisible(true);
		}
		return match;
	}
	
	public void cleanFilter(){
		setVisible(true);
	}
}
