package fr.aliasource.webmail.client.composer.contacts;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.HandlesAllKeyEvents;
import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.ScrollEvent;
import com.google.gwt.event.dom.client.ScrollHandler;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Widget;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.addressbook.ContactGroupAsyncCallback;
import fr.aliasource.webmail.client.ctrl.AjaxCall;
import fr.aliasource.webmail.client.shared.ContactGroup;
import fr.aliasource.webmail.client.shared.book.UiDisplayEmail;
import fr.aliasource.webmail.client.shared.book.UiPagingEmails;
import fr.aliasource.webmail.client.widget.MinigTextBox;
import fr.aliasource.webmail.client.widget.RemoveRecipientEvent;
import fr.aliasource.webmail.client.widget.RemoveRecipientHandler;
import fr.aliasource.webmail.client.widget.SimpleRecipientsPanel;

public class ChooseContactsPanel extends Composite {

	private static final String SEARCH_GROUP_VALUE = "searchGroupContact";
	private static final int SEARCH_GROUP_INDEX = 0;
	private static final int PAGE_SIZE = 100;

	private View ui;
	private ContactGroup groupSelected;
	private Map<String, ContactGroup> listGroup;
	private Map<UiDisplayEmail, ChooseContactWidget> chooseContacts;
	private int contactsDisplay;
	private int totalContacts;

	@UiField
	MinigTextBox searchTextBox;
	@UiField
	ListBox groupListBox;
	@UiField
	HTMLPanel contactsListPanel;
	@UiField
	SimpleRecipientsPanel chosenContactsRecipientsPanel;
	@UiField
	ScrollPanel scrollPanel;

	interface Binder extends UiBinder<Widget, ChooseContactsPanel> {
	}

	private static final Binder BINDER = GWT.create(Binder.class);

	public ChooseContactsPanel(View ui) {
		initWidget(BINDER.createAndBindUi(this));
		this.groupSelected = ContactGroup.DefaultGroup.All.getGroup();
		this.listGroup = new HashMap<String, ContactGroup>();
		this.chooseContacts = new HashMap<UiDisplayEmail, ChooseContactWidget>();
		this.ui = ui;
		this.contactsDisplay = 0;
		this.totalContacts = 0;
		initGroups();
		initContacts();
		initSearchTextBox();

		chosenContactsRecipientsPanel
				.addRemoveRecipientHandler(new RemoveRecipientHandler() {
					@Override
					public void onRemoveRecipient(RemoveRecipientEvent event) {
						ChooseContactWidget wid = chooseContacts.get(event
								.getEmail());
						if (wid != null) {
							wid.setSeleted(false);
						}
					}
				});
		scrollPanel.addScrollHandler(new ScrollHandler() {
			@Override
			public void onScroll(ScrollEvent event) {
				
				int finalPos = scrollPanel.getScrollPosition() + scrollPanel.getOffsetHeight(); 
				int size = DOM.getElementPropertyInt(scrollPanel.getElement(),
						"scrollHeight");
				if (finalPos + 100 > size) {
					loadNewContactPage();
				}
			}
		});
	}

	private void initGroups() {
		ui.getSpinner().startSpinning();

		AsyncCallback<ContactGroup[]> ac = new ContactGroupAsyncCallback() {
			@Override
			public void onFailure(Throwable caught) {
				ui.log("contactGroups failure");
				ui.getSpinner().stopSpinning();
			}

			@Override
			public void onSuccess(Set<ContactGroup> groups) {
				updateGroupListBox(groups);
				ui.getSpinner().stopSpinning();
			}
		};
		AjaxCall.contacts.getContactGroups(ac);

		groupListBox.addChangeHandler(new ChangeHandler() {
			@Override
			public void onChange(ChangeEvent arg0) {
				searchTextBox.clear();
				int ind = groupListBox.getSelectedIndex();
				String groupKey = groupListBox.getValue(ind);
				ContactGroup cont = listGroup.get(groupKey);
				if (cont == null
						|| SEARCH_GROUP_VALUE.equals(groupListBox
								.getValue(groupListBox.getSelectedIndex()))) {
					cont = ContactGroup.DefaultGroup.All.getGroup();
				}
				groupSelected = cont;
				initContacts();
			}
		});
	}

	private void updateGroupListBox(Set<ContactGroup> groups) {
		this.groupListBox.clear();
		this.listGroup.clear();
		for (ContactGroup group : groups) {
			this.groupListBox.addItem(group.getDisplayName(), group.getId());
			listGroup.put(group.getId(), group);
		}

	}

	private void initSearchTextBox() {
		searchTextBox.setDescription(I18N.strings.searchContacts());
		FilterFieldHandlers fil = new FilterFieldHandlers();
		searchTextBox.addAllKeyHandler(fil);
		searchTextBox.addValueChangeHandler(fil);
	}

	private void initContacts() {
		ui.getSpinner().startSpinning();
		this.contactsDisplay = 0;
		this.totalContacts = 0;
		AsyncCallback<UiPagingEmails> ac = new AsyncCallback<UiPagingEmails>() {
			public void onFailure(Throwable caught) {
				ui.log("loadGroup failure");
				ui.getSpinner().stopSpinning();
			}

			public void onSuccess(UiPagingEmails contacts) {
				contactsListPanel.clear();
				displayContacts(contacts);
				scrollPanel.scrollToTop();
				ui.getSpinner().stopSpinning();
			}
		};
		doQuery(ac);
	}

	private void loadNewContactPage() {
		AsyncCallback<UiPagingEmails> ac = new AsyncCallback<UiPagingEmails>() {
			public void onFailure(Throwable caught) {
				ui.log("loadGroup failure");
				ui.getSpinner().stopSpinning();
			}

			public void onSuccess(UiPagingEmails contacts) {
				displayContacts(contacts);
				ui.getSpinner().stopSpinning();
			}
		};
		doQuery(ac);
	}

	private void doQuery(AsyncCallback<UiPagingEmails> ac) {
		int limit = contactsDisplay + PAGE_SIZE;
		AjaxCall.pagingContacts.getPagingEmails(groupSelected,
				searchTextBox.getValue(), contactsDisplay, limit, ac);
	}

	private void displayContacts(UiPagingEmails page) {
		this.totalContacts = page.getTotalCount();
		this.contactsDisplay = page.getLimit();
		if (searchGroupSelectedExist()) {
			groupListBox.setItemText(SEARCH_GROUP_INDEX,
					I18N.strings.searchContactsGroupName());
		}
		Set<UiDisplayEmail> selectedRecip = getRecipients();
		for (final UiDisplayEmail c : page.getContactsPage()) {
			ChooseContactWidget w = chooseContacts.get(c);
			if (w == null) {
				w = createChooseContactWidget(c);
			}
			w.cleanFilter();
			if (selectedRecip.contains(c)) {
				w.setSeleted(true);
			}
			contactsListPanel.add(w, contactsListPanel.getElement());
		}
	}

	private ChooseContactWidget createChooseContactWidget(final UiDisplayEmail c) {
		ChooseContactWidget w = new ChooseContactWidget(c);

		w.addSelectContactHandler(new SelectContactHandler() {
			@Override
			public void onSelectContact(SelectContactEvent event) {
				if (SelectContactEvent.Selection.Select.equals(event
						.getSelection())) {
					chosenContactsRecipientsPanel.addRecipient(event.getEmail());
				} else {
					chosenContactsRecipientsPanel.removeRecipient(event
							.getEmail());
				}
			}
		});
		chooseContacts.put(c, w);
		return w;
	}

	private class FilterFieldHandlers extends HandlesAllKeyEvents implements
			ValueChangeHandler<String> {
		private String lastFilter = "";

		@Override
		public void onKeyDown(KeyDownEvent event) {
			if (!lastFilter.equalsIgnoreCase(searchTextBox.getValue())) {
				lastFilter = searchTextBox.getValue();
				applyFilter();
			}
		}

		@Override
		public void onKeyUp(KeyUpEvent event) {
			if (!lastFilter.equalsIgnoreCase(searchTextBox.getValue())) {
				lastFilter = searchTextBox.getValue();
				applyFilter();
			}
		}

		@Override
		public void onKeyPress(KeyPressEvent event) {
			if (!lastFilter.equalsIgnoreCase(searchTextBox.getValue())) {
				lastFilter = searchTextBox.getValue();
				applyFilter();
			}
		}

		@Override
		public void onValueChange(ValueChangeEvent<String> event) {
			if (!lastFilter.equalsIgnoreCase(searchTextBox.getValue())) {
				lastFilter = searchTextBox.getValue();
				applyFilter();
			}
		}
	}

	private void applyFilter() {
		if (searchTextBox.getValue().isEmpty()) {
			if (searchGroupSelectedExist()) {
				groupListBox.removeItem(SEARCH_GROUP_INDEX);
			}
		} else {
			if (!searchGroupSelectedExist()) {
				groupListBox.insertItem(I18N.strings.searchContactsGroupName(),
						SEARCH_GROUP_VALUE, SEARCH_GROUP_INDEX);
			}
			groupListBox.setItemSelected(SEARCH_GROUP_INDEX, true);
		}
		if (this.contactsDisplay < totalContacts && totalContacts != 0) {
			initContacts();
		} else {
			for (int i = 0; i < contactsListPanel.getWidgetCount(); i++) {
				ChooseContactWidget wid = (ChooseContactWidget) contactsListPanel
						.getWidget(i);
				wid.applyFilter(searchTextBox.getValue());
			}
		}
	}

	private boolean searchGroupSelectedExist() {
		return groupListBox.getItemCount() > 0
				&& SEARCH_GROUP_VALUE.equals(groupListBox.getValue(0));
	}

	public Set<UiDisplayEmail> getRecipients() {
		return this.chosenContactsRecipientsPanel.getRecipients();
	}

	public void setRecipients(Set<UiDisplayEmail> emails) {
		chosenContactsRecipientsPanel.setRecipients(emails);
		for (UiDisplayEmail e : emails) {
			ChooseContactWidget wid = chooseContacts.get(e);
			if (wid != null) {
				wid.setSeleted(true);
			}
		}
	}

	public void clean() {
		chooseContacts.clear();
		chosenContactsRecipientsPanel.clean();
		initContacts();
	}
}
