package fr.aliasource.webmail.client.shared.book;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import fr.aliasource.webmail.client.shared.DisplayNameUtils;
import fr.aliasource.webmail.client.shared.mailing.MailingListsUtils;

public class UiDisplayEmail implements Serializable, Comparable<UiDisplayEmail> {
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 3957495735636429210L;
	
	private String displayName;
	
	//Case of mailing lists
	private List<String> emails = new ArrayList<String>();
	private boolean mailingList = false;
	
	public UiDisplayEmail(){
	}
	
	public UiDisplayEmail(String displayName, String email){
		this.displayName = displayName;
		emails.add(email);
	}
	
	public UiDisplayEmail(String firstName, String lastName, String email){
		
		displayName = DisplayNameUtils.formatName(firstName, lastName, email);
		emails.add(email);
	}

	public UiDisplayEmail(String displayName, List<String> emails) {
		this.displayName = displayName;
		this.emails = emails;
		this.mailingList = true;
	}

	//Should not be call on server side
	public String getDisplayName() {
		if(isMailingList()){
			return MailingListsUtils.getDisplayName(displayName);
		}
		
		
		return displayName;
	}
	
	

	/**
	 * @return the emails
	 */
	public List<String> getEmails() {
		return emails;
	}

	/**
	 * @return the mailingList
	 */
	public boolean isMailingList() {
		return mailingList;
	}

	

	@Override
	public int compareTo(UiDisplayEmail o) {
		String val = displayName+" "+getEmail();
		String val2 = o.displayName+" "+o.getEmail();
		return val.compareToIgnoreCase(val2);
	}

	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "UiDisplayEmail [displayName=" + displayName + ", emails="
				+ emails + ", mailingList=" + mailingList + "]";
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((displayName == null) ? 0 : displayName.hashCode());
		result = prime * result + ((emails == null) ? 0 : emails.hashCode());
		result = prime * result + (mailingList ? 1231 : 1237);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		UiDisplayEmail other = (UiDisplayEmail) obj;
		if (displayName == null) {
			if (other.displayName != null)
				return false;
		} else if (!displayName.equals(other.displayName))
			return false;
		if (emails == null) {
			if (other.emails != null)
				return false;
		} else if (!emails.equals(other.emails))
			return false;
		if (mailingList != other.mailingList)
			return false;
		return true;
	}

	public String getEmail() {
		if(emails!=null && emails.size()==1){
			return emails.get(0);
		}
		return "";
	}

}
