package fr.aliasource.webmail.client.widget;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.BlurHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.HandlesAllKeyEvents;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

public class MinigTextBox extends Composite {

	private final static String CSS_HINT = "showHint";

	interface Binder extends UiBinder<Widget, MinigTextBox> {
	}

	private static final Binder BINDER = GWT.create(Binder.class);

	@UiField
	TextBox searchTextBox;
	@UiField
	ClearButton clearSearchTextBox;

	private String desciption = "";

	public MinigTextBox() {
		super();
		initWidget(BINDER.createAndBindUi(this));
		initSearchTextBox();
		initClearSearchTextBox();
	}

	private void initSearchTextBox() {
		searchTextBox.addStyleName(CSS_HINT);
		searchTextBox.addFocusHandler(new FocusHandler() {
			@Override
			public void onFocus(FocusEvent arg0) {
				searchTextBox.removeStyleName(CSS_HINT);
				if (desciption.equals(searchTextBox.getText())) {
					searchTextBox.setText("");
				}
			}
		});

		searchTextBox.addBlurHandler(new BlurHandler() {
			@Override
			public void onBlur(BlurEvent arg0) {
				if (searchTextBox.getText().isEmpty()) {
					clear();
				}
			}
		});
	}

	private void initClearSearchTextBox() {
		clearSearchTextBox.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent arg0) {
				clear();
			}
		});
	}

	public void setDescription(String description) {
		boolean needClear = searchTextBox.getText().isEmpty()
				|| searchTextBox.getText().equals(this.desciption);
		this.desciption = description;
		if (needClear) {
			clear();
		}
	}

	public void clear() {
		searchTextBox.setText(desciption);
		searchTextBox.addStyleName(CSS_HINT);
		ValueChangeEvent.fire(searchTextBox, "");
	}

	public void addAllKeyHandler(HandlesAllKeyEvents handler) {
		handler.addKeyHandlersTo(searchTextBox);
	}

	public String getValue() {
		if (searchTextBox.getText().equals(this.desciption)) {
			return "";
		}
		return searchTextBox.getText();
	}
	
	
	public void setText(String text){
		searchTextBox.setText(text);
	}

	public void addValueChangeHandler(ValueChangeHandler<String> fil) {
		searchTextBox.addValueChangeHandler(fil);
	}

	public void addKeyUpHandler(KeyUpHandler keyUpHandler) {
		searchTextBox.addKeyUpHandler(keyUpHandler);
	}
}
