/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.client.widget;

import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyDownHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;

import fr.aliasource.webmail.client.shared.book.UiDisplayEmail;


/**
 * The address selection widget used in mail composer
 * 
 * @author adrien
 * 
 */
public class SimpleRecipientsPanel extends ScrollPanel implements HasRemoveRecipientHandlers{
	private final String CSS_REMOVE_RECIPIENT = "removeRecipient";
	
	private TextBox mailField;
	private FlowPanel recipListPanel;
	private int token;
	private int recipCount;
	private HashMap<UiDisplayEmail, RecipientWidget> recipients;
	private long lastTs;
	private static String PATTERN_EMAIL = "[a-zA-Z_0-9\\-.]+@[a-zA-Z_0-9\\-.]+\\.[a-z]+";
	
	public SimpleRecipientsPanel() {
		super();
		token = -1;
		mailField = new TextBox();
		createMailFieldKeyboardListener();

		FlowPanel wrap = new FlowPanel();
		wrap.setStyleName("wrap");
		recipListPanel = new FlowPanel();
		wrap.add(recipListPanel);
		wrap.add(mailField);
		add(wrap);
		setStyleName("simpleEnveloppeField");
		setWidth("100%");

		recipients = new HashMap<UiDisplayEmail, RecipientWidget>();
		// Add click listener
		sinkEvents(Event.ONCLICK);
	}

	public void onBrowserEvent(Event event) {
		switch (DOM.eventGetType(event)) {
		case Event.ONCLICK:
			mailField.setFocus(true);
			break;
		case Event.ONBLUR:
			break;
		}
		
	}
	
	public void addRecipient(UiDisplayEmail email) {
		createRecipient(email);
	}
	
	private void addRecipient(String email) {
		createRecipient(new UiDisplayEmail(email, email));
	}

	private void createRecipient(UiDisplayEmail mail) {
		final RecipientWidget recip = new RecipientWidget(mail);
		recip.addMouseOverHandler(new MouseOverHandler() {
			@Override
			public void onMouseOver(MouseOverEvent arg0) {
				recip.addStyleName(CSS_REMOVE_RECIPIENT);
			}
		});
		
		recip.addMouseOutHandler(new MouseOutHandler() {
			@Override
			public void onMouseOut(MouseOutEvent arg0) {
				recip.removeStyleName(CSS_REMOVE_RECIPIENT);
			}
		});
		
		recip.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent arg0) {
				removeRecipient((RecipientWidget) arg0.getSource());
			}
		});
		recipListPanel.add(recip);
		scrollToBottom();
		recipients.put(mail, recip);
	}
	
	public void removeRecipient(UiDisplayEmail email) {
		RecipientWidget wid = recipients.get(email);
		if(wid != null){
			removeRecipient(wid);
		}
	}

	protected void removeRecipient(RecipientWidget recip) {
		recipients.remove(recip.getEmail());
		recipListPanel.remove(recip);
		fireEvent(new RemoveRecipientEvent(recip.getEmail()));
	}
	
	

	private void createMailFieldKeyboardListener() {
		mailField.addKeyDownHandler(new KeyDownHandler() {
			public void onKeyDown(KeyDownEvent kde) {
				long ts = new Date().getTime();
				long diff = ts - lastTs;
				lastTs = ts;
				if (diff < 150) {
					// crap to prevent double delete on backspace...
					return;
				}
				if (mailField.getText().isEmpty()) {
					recipCount = recipListPanel.getWidgetCount();
					switch (kde.getNativeKeyCode()) {
					case KeyCodes.KEY_BACKSPACE:
						if (recipCount > 0) {
							RecipientWidget recip;
							if (token == -1) {
								recip = (RecipientWidget) recipListPanel.getWidget(recipCount - 1);
								removeRecipient(recip);
							} else {
								recip = (RecipientWidget) recipListPanel.getWidget(token);
							}
							removeRecipient(recip);
							mailField.setFocus(true);
							token = -1;
						}
						break;
					case KeyCodes.KEY_LEFT:
						if (recipCount > 0) {
							if (token == -1) {
								token = recipCount - 1;
							} else if (token > 0) {
								recipListPanel.getWidget(token)
										.removeStyleName("highlightRecipient");
								token--;
							} else {
								token = 0;
							}
							recipListPanel.getWidget(token).addStyleName(
									"highlightRecipient");
						}
						break;
					case KeyCodes.KEY_RIGHT:
						if (recipCount > 0) {
							if (token < recipCount - 1 && token > -1) {
								recipListPanel.getWidget(token)
										.removeStyleName("highlightRecipient");
								token++;
							} else {
								token = recipCount - 1;
							}
							recipListPanel.getWidget(token).addStyleName(
									"highlightRecipient");
						}
						break;
					case KeyCodes.KEY_DELETE:
						if (token >= 0) {
							String email = recipListPanel.getWidget(token)
									.getTitle();
							recipListPanel.remove(token);
							recipients.remove(email);
							mailField.setFocus(true);
							recipCount = recipListPanel.getWidgetCount();
							if (recipCount > 0) {
								if (token >= recipCount - 1) {
									token = recipCount - 1;
								} else if (token <= 0) {
									token = 0;
								}
								recipListPanel.getWidget(token).addStyleName(
										"highlightRecipient");
							} else {
								token = -1;
							}
						}
						break;
					case KeyCodes.KEY_ESCAPE:
						if (token >= 0) {
							recipListPanel.getWidget(token).removeStyleName(
									"highlightRecipient");
							mailField.setFocus(true);
						}
						token = -1;
						break;
					default:
						if (token > -1) {
							recipListPanel.getWidget(token).removeStyleName(
									"highlightRecipient");
							token = -1;
						}
						break;
					}
				} else {
					String email = mailField.getText();
					switch (kde.getNativeKeyCode()) {
					case KeyCodes.KEY_ENTER:
						addEmail(email);
						break;
					case KeyCodes.KEY_ESCAPE:
						mailField.setText("");
						break;
					}
				}
			}
		});
	}

	public void clearText() {
		recipListPanel.clear();
		recipients.clear();
	}

	public void focus() {
		mailField.setFocus(true);
	}

	private void addEmail(String email) {
		if (email.matches(PATTERN_EMAIL)) {
			addRecipient(email);
			mailField.setFocus(true);
			mailField.setText("");
		} else if (email.contains(",") || email.contains("<")) {
			String[] spl = email.split(",");
			for (String s : spl) {
				String m = s.trim();
				int idx = m.indexOf("<");
				if (idx > 0) {
					m = m.substring(idx + 1);
					m = m.replace(">", "");
					m = m.replace("\"", "");
				}
				if (m.matches(PATTERN_EMAIL)) {
					addRecipient(m);
				}
			}
			mailField.setFocus(true);
			mailField.setText("");
		}
	}
	
	@Override
	public void addRemoveRecipientHandler(RemoveRecipientHandler handler) {
		addHandler(handler, RemoveRecipientEvent.getType());
	}
	
	public boolean hasRecipent(UiDisplayEmail email){
		return this.recipients.containsKey(email);
	}
	
	public Set<UiDisplayEmail> getRecipients(){
		return recipients.keySet();
	}
	
	public void setRecipients(Set<UiDisplayEmail> emails){
		for(UiDisplayEmail e : emails){
			addRecipient(e);
		}
	}

	public void clean() {
		mailField.setText("");
		recipListPanel.clear();
		recipients.clear();
	}
}
