/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.server.proxy.client.http;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.commons.httpclient.HttpClient;
import org.obm.sync.book.BookItemsParser;
import org.obm.sync.book.Contact;
import org.obm.sync.book.Email;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fr.aliasource.webmail.client.shared.ContactGroup;
import fr.aliasource.webmail.client.shared.book.UiDisplayEmail;
import fr.aliasource.webmail.client.shared.book.UiPagingEmails;
import fr.aliasource.webmail.client.shared.mailing.MailingListsUtils;

public class GetPagingEmailsMethod extends AbstractClientMethod {

	private String token;
	private String cachedQuery;
	private ContactGroup cachedGroup;
	private List<UiDisplayEmail> cachedEmail;

	public GetPagingEmailsMethod(HttpClient hc, String token, String backendUrl) {
		super(hc, backendUrl, "/contacts.do");
		this.token = token;
		cachedEmail = new ArrayList<UiDisplayEmail>(0);
	}
	
	public UiPagingEmails getPagingEmails(ContactGroup cg, String query,
			Integer start, Integer limit) {
		UiPagingEmails ret = new UiPagingEmails(start, limit);
		cacheContacts(cg, query);
		ret.setTotalCount(cachedEmail.size());
		if (start > cachedEmail.size()) {
			ret.setContactsPage(new ArrayList<UiDisplayEmail>(0));
		} else {
			 if (limit > cachedEmail.size()) {
					limit = cachedEmail.size();
			 }
			 ret.setContactsPage(cachedEmail.subList(start, limit));
		}
		
		return ret;
	}

	private void cacheContacts(ContactGroup cg, String query) {
		if (query == null) {
			query = "";
		}
		if (cachedGroup == null) {
			cachedGroup = cg;
		}
		if (!query.equals(cachedQuery) || !cg.equals(cachedGroup)) {
			Map<String, String> params = new HashMap<String, String>();
			params.put("token", token);
			params.put("group", "" + cg.getId());
			params.put("origin", "" + cg.getOrigin().asString());
			if (query != null) {
				params.put("query", query);
			}
			Document doc = execute(params);
			if (doc != null) {
				if (logger.isDebugEnabled()) {
					DOMUtils.logDom(doc);
				}
				BookItemsParser parser = new BookItemsParser();

				Element root = doc.getDocumentElement();
				NodeList contactNodes = root.getElementsByTagName("contact");
				SortedSet<UiDisplayEmail> cache = new TreeSet<UiDisplayEmail>();
				for (int i = 0; i < contactNodes.getLength(); i++) {
					Element cn = (Element) contactNodes.item(i);
					Contact obm = parser.parseContact(cn);
					Set<UiDisplayEmail> ui = toUi(obm);
					cache.addAll(ui);
				}
				cachedEmail.clear();
				cachedEmail.addAll(cache);
			}
		}
		this.cachedQuery = query;
		this.cachedGroup = cg;
	}

	private Set<UiDisplayEmail> toUi(Contact c) {
		Set<UiDisplayEmail> ret = new HashSet<UiDisplayEmail>();
		if(MailingListsUtils.isMailingList(c.getLastname())){
			List<String> emails = new ArrayList<String>();
			for (Email e : c.getEmails().values()) {
				if (e.getEmail() != null && !e.getEmail().isEmpty()) {
					emails.add(e
							.getEmail());
				}
			}
			ret.add(new UiDisplayEmail(c.getFirstname() ,emails));
		} else {
			
			for (Email e : c.getEmails().values()) {
				if (e.getEmail() != null && !e.getEmail().isEmpty()) {
					ret.add(new UiDisplayEmail(c.getFirstname(), c.getLastname(), e
							.getEmail()));
				}
			}
			
		}
		return ret;
	}
}
