package fr.aliasource.webmail.server.proxy.client.http.filters;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.obm.sync.items.AbstractItemsParser;
import org.obm.sync.utils.DOMUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import fr.aliasource.webmail.client.shared.EmailAddress;
import fr.aliasource.webmail.client.shared.book.UIMailingList;

public class MailingListItemsParser extends AbstractItemsParser {

	@SuppressWarnings("unused")
	private static final Log logger = LogFactory
			.getLog(MailingListItemsParser.class);

	public UIMailingList parseMailingList(String parameter) throws SAXException,
			IOException, ParserConfigurationException,
			FactoryConfigurationError {
		Document doc = DOMUtils.parse(new ByteArrayInputStream(parameter
				.getBytes()));

		Element root = doc.getDocumentElement();
		return parseMailingList(root);
	}

	public List<UIMailingList> parseListMailingList(Document doc) {
		Element documentElement = doc.getDocumentElement();
		List<UIMailingList> ret = new LinkedList<UIMailingList>();
		NodeList mlsNodeList = documentElement
				.getElementsByTagName("mailingList");
		for (int i = 0; i < mlsNodeList.getLength(); i++) {
			Element e = (Element) mlsNodeList.item(i);
			UIMailingList ml = parseMailingList(e);
			ret.add(ml);
		}
		return ret;
	}

	public UIMailingList parseMailingList(Element root) {
		Element mlEle = root;
		if(!mlEle.getTagName().equalsIgnoreCase("mailingList")){
			mlEle = DOMUtils.getUniqueElement(root, "mailingList");	
		}
		UIMailingList ml = new UIMailingList();
		String id = mlEle.getAttribute("id");
		if (id != null && id.length() > 0) {
			try {
				ml.setId(Integer.valueOf(id));
			} catch (NumberFormatException e) {
			}
		}
		ml.setName(mlEle.getAttribute("name"));
		ml.addEmails(parseEmails(mlEle));
		return ml;
	}

	private List<EmailAddress> parseEmails(Element uniqueElement) {
		String[] attrs = {"id", "label", "address" };
		String[][] values = DOMUtils
				.getAttributes(uniqueElement, "email", attrs);
		List<EmailAddress> ret = new ArrayList<EmailAddress>(values.length);
		for (String[] p : values) {
			EmailAddress e = new EmailAddress(p[1]+" <"+p[2]+">", p[2]);
			String id = p[0];
			if (id != null && id.length() > 0) {
				try {
					e.setId(Integer.valueOf(id));
				} catch (NumberFormatException x) {
				}
			}
			ret.add(e);
		}
		return ret;
	}

	public List<EmailAddress> parseMailingListEmails(String mailingListEmails) {
		try {
			Document doc = DOMUtils.parse(new ByteArrayInputStream(mailingListEmails
					.getBytes()));
			return parseEmails(doc.getDocumentElement());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new ArrayList<EmailAddress>(0);
	}
	
	public List<EmailAddress> parseMailingListEmails(Document doc) {
		return parseEmails(doc.getDocumentElement());
	}
}
