/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.server.proxy.client.http.mailinglist;

import java.util.List;

import org.apache.commons.httpclient.HttpClient;

import fr.aliasource.webmail.client.shared.EmailAddress;
import fr.aliasource.webmail.client.shared.book.UIMailingList;

public class MailingListMethods implements IMailingListMethods{
	
	private ListAllMailingListMethod listAllMailingListMethod;
	private CreateMailingListMethod createMailingListMethod;
	private ModifyMailingListMethod modifyMailingListMethod;
	private RemoveMailingListMethod removeMailingListMethod;
	private GetMailingListFromIdMethod getMailingListFromIdMethod;
	private AddEmailsMethod addEmailsMethod;
	private RemoveEmailMethod removeEmailMethod;
	
	public MailingListMethods(HttpClient hc, String token, String backendUrl) {
		listAllMailingListMethod = new ListAllMailingListMethod(hc,token, backendUrl);
		createMailingListMethod = new CreateMailingListMethod(hc,token, backendUrl);
		modifyMailingListMethod = new ModifyMailingListMethod(hc,token, backendUrl);
		removeMailingListMethod = new RemoveMailingListMethod(hc,token, backendUrl);
		getMailingListFromIdMethod = new GetMailingListFromIdMethod(hc,token, backendUrl);
		addEmailsMethod = new AddEmailsMethod(hc,token, backendUrl);
		removeEmailMethod = new RemoveEmailMethod(hc,token, backendUrl);
	}

	public List<UIMailingList> listAllMailingList() {
		return listAllMailingListMethod.listAllMailingList();
	}

	public UIMailingList createMailingList(UIMailingList mailingList) {
		return createMailingListMethod.createMailingList(mailingList);
	}

	public UIMailingList modifyMailingList(UIMailingList mailingList) {
		return modifyMailingListMethod.modifyMailingList(mailingList);
	}

	public void removeMailingList(Integer id) {
		removeMailingListMethod.removeMailingList(id);
	}

	public UIMailingList getMailingListFromId(Integer id) {
		return getMailingListFromIdMethod.getMailingListFromId(id);
	}

	public List<EmailAddress> addEmails(Integer mailingListId,
			List<EmailAddress> email) {
		return addEmailsMethod.addEmails(mailingListId, email);
	}

	public void removeEmail(Integer mailingListId, Integer emailId) {
		removeEmailMethod.removeEmail(mailingListId, emailId);
	}
}
